var spawn     = require('child_process').spawn,
    os_name   = process.platform.replace('win32', 'windows').replace('darwin', 'mac'),
    os_module = require(require('path').join(__dirname, os_name)),
    Emitter   = require('events').EventEmitter;

var child,
    stopped,
    watchers  = {};

var watch = (trigger) => {
  return new Promise((resolve, reject) => {

    var watcher,
        error;
        stopped = false;
  
    if (!trigger)
      reject(new Error('Trigger name required'));

    // Create child process if doesn't exist
    if (!child) child = spawn(os_module.command[0], os_module.command.slice(1));

    var done = (exit) => {
      if (stopped) {
        watchers = {};
        reject(new Error('Triggers already stopped'));
      }

      if (exit) {
        error = new Error('Unable to start watcher command.');
        child = null;
      } else resolve(emitter);

      if (error) {
        reject(error);
      }
    }

    child.on('error', (err) => {
      if (err.code == 'ENOENT')
        err.message = 'ENOENT - Command not found: ' + os_module.command;
      error = err;
    })

    child.on('exit', (code) => {
      child = null;
      done(true);
    })

    if (!watchers[trigger]) {
      var emitter = new Emitter();
      watcher = new os_module.Watcher(child, trigger, emitter);
      watchers[trigger] = watcher;
    }

    setTimeout(done, 100);
  });

}

var unwatch = (trigger) => {
  // if there are no watchers, return
  if (Object.keys(watchers).length == 0) return;

  // find watcher and stop it
  var watcher = watchers[trigger];

  if (!watcher) return;

  watcher.stop();
  delete watchers[trigger];

  // console.log('Unwatched ' + trigger + '. Current: ' + Object.keys(watchers).length);

  // if no watchers are left, stop process
  if (child && Object.keys(watchers).length == 0) {
    // console.log('No more watchers, so killing child.');
    stopped = true;
    child.kill();
  }

}

module.exports.watch = watch;
module.exports.unwatch = unwatch;

module.exports.stop = () => {
  for (var trigger in watchers)
    module.exports.unwatch(trigger);
}